@@ -11,7 +11,7 @@ import py
 import tox
 from tox import reporter
 from tox.action import Action
-from tox.config.parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
+from tox.config.parallel import ENV_VAR_KEY_PRIVATE as PARALLEL_ENV_VAR_KEY_PRIVATE
 from tox.constants import INFO, PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
 from tox.package.local import resolve_package
 from tox.util.lock import get_unique_file
@@ -23,7 +23,7 @@ from .config import DepConfig
 class CreationConfig:
     def __init__(
         self,
-        base_resolved_python_md5,
+        base_resolved_python_sha256,
         base_resolved_python_path,
         tox_version,
         sitepackages,
@@ -31,7 +31,7 @@ class CreationConfig:
         deps,
         alwayscopy,
     ):
-        self.base_resolved_python_md5 = base_resolved_python_md5
+        self.base_resolved_python_sha256 = base_resolved_python_sha256
         self.base_resolved_python_path = base_resolved_python_path
         self.tox_version = tox_version
         self.sitepackages = sitepackages
@@ -41,9 +41,12 @@ class CreationConfig:
 
     def writeconfig(self, path):
         lines = [
-            "{} {}".format(self.base_resolved_python_md5, self.base_resolved_python_path),
+            "{} {}".format(self.base_resolved_python_sha256, self.base_resolved_python_path),
             "{} {:d} {:d} {:d}".format(
-                self.tox_version, self.sitepackages, self.usedevelop, self.alwayscopy
+                self.tox_version,
+                self.sitepackages,
+                self.usedevelop,
+                self.alwayscopy,
             ),
         ]
         for dep in self.deps:
@@ -64,11 +67,11 @@ class CreationConfig:
             alwayscopy = bool(int(alwayscopy))
             deps = []
             for line in lines:
-                base_resolved_python_md5, depstring = line.split(None, 1)
-                deps.append((base_resolved_python_md5, depstring))
-            base_resolved_python_md5, base_resolved_python_path = base_resolved_python_info
+                base_resolved_python_sha256, depstring = line.split(None, 1)
+                deps.append((base_resolved_python_sha256, depstring))
+            base_resolved_python_sha256, base_resolved_python_path = base_resolved_python_info
             return CreationConfig(
-                base_resolved_python_md5,
+                base_resolved_python_sha256,
                 base_resolved_python_path,
                 tox_version,
                 sitepackages,
@@ -81,7 +84,7 @@ class CreationConfig:
 
     def matches_with_reason(self, other, deps_matches_subset=False):
         for attr in (
-            "base_resolved_python_md5",
+            "base_resolved_python_sha256",
             "base_resolved_python_path",
             "tox_version",
             "sitepackages",
@@ -119,7 +122,7 @@ class VirtualEnv(object):
     def new_action(self, msg, *args):
         config = self.envconfig.config
         command_log = self.env_log.get_commandlog(
-            "test" if msg in ("run-test", "run-test-pre", "run-test-post") else "setup"
+            "test" if msg in ("run-test", "run-test-pre", "run-test-post") else "setup",
         )
         return Action(
             self.name,
@@ -130,13 +133,18 @@ class VirtualEnv(object):
             command_log,
             self.popen,
             self.envconfig.envpython,
+            self.envconfig.suicide_timeout,
+            self.envconfig.interrupt_timeout,
+            self.envconfig.terminate_timeout,
         )
 
     def get_result_json_path(self):
         if self._result_json_path is None:
             if self.envconfig.config.option.resultjson:
                 self._result_json_path = get_unique_file(
-                    self.path, PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
+                    self.path,
+                    PARALLEL_RESULT_JSON_PREFIX,
+                    PARALLEL_RESULT_JSON_SUFFIX,
                 )
         return self._result_json_path
 
@@ -146,7 +154,7 @@ class VirtualEnv(object):
 
     @property
     def path(self):
-        """ Path to environment base dir. """
+        """Path to environment base dir."""
         return self.envconfig.envdir
 
     @property
@@ -155,18 +163,18 @@ class VirtualEnv(object):
 
     @property
     def name(self):
-        """ test environment name. """
+        """test environment name."""
         return self.envconfig.envname
 
     def __repr__(self):
         return "<VirtualEnv at {!r}>".format(self.path)
 
     def getcommandpath(self, name, venv=True, cwd=None):
-        """ Return absolute path (str or localpath) for specified command name.
+        """Return absolute path (str or localpath) for specified command name.
 
         - If it's a local path we will rewrite it as as a relative path.
         - If venv is True we will check if the command is coming from the venv
-          or is whitelisted to come from external.
+          or is allowed to come from external.
         """
         name = str(name)
         if os.path.isabs(name):
@@ -177,18 +185,18 @@ class VirtualEnv(object):
                 return str(path)
 
         if venv:
-            path = self._venv_lookup_and_check_external_whitelist(name)
+            path = self._venv_lookup_and_check_external_allowlist(name)
         else:
             path = self._normal_lookup(name)
 
         if path is None:
             raise tox.exception.InvocationError(
-                "could not find executable {}".format(pipes.quote(name))
+                "could not find executable {}".format(pipes.quote(name)),
             )
 
         return str(path)  # will not be rewritten for reporting
 
-    def _venv_lookup_and_check_external_whitelist(self, name):
+    def _venv_lookup_and_check_external_allowlist(self, name):
         path = self._venv_lookup(name)
         if path is None:
             path = self._normal_lookup(name)
@@ -209,10 +217,11 @@ class VirtualEnv(object):
                 "  cmd: {}\n"
                 "  env: {}\n"
                 "Maybe you forgot to specify a dependency? "
-                "See also the whitelist_externals envconfig setting.\n\n"
+                "See also the allowlist_externals envconfig setting.\n\n"
                 "DEPRECATION WARNING: this will be an error in tox 4 and above!".format(
-                    path, self.envconfig.envdir
-                )
+                    path,
+                    self.envconfig.envdir,
+                ),
             )
 
     def is_allowed_external(self, p):
@@ -220,15 +229,24 @@ class VirtualEnv(object):
         if tox.INFO.IS_WIN:
             tryadd += [os.path.normcase(x) for x in os.environ["PATHEXT"].split(os.pathsep)]
             p = py.path.local(os.path.normcase(str(p)))
-        for x in self.envconfig.whitelist_externals:
+
+        if self.envconfig.allowlist_externals and self.envconfig.whitelist_externals:
+            raise tox.exception.ConfigError(
+                "Either whitelist_externals or allowlist_externals might be specified, not both",
+            )
+
+        allowed_externals = (
+            self.envconfig.whitelist_externals or self.envconfig.allowlist_externals
+        )
+        for x in allowed_externals:
             for add in tryadd:
                 if p.fnmatch(x + add):
                     return True
         return False
 
     def update(self, action):
-        """ return status string for updating actual venv to match configuration.
-            if status string is empty, all is ok.
+        """return status string for updating actual venv to match configuration.
+        if status string is empty, all is ok.
         """
         rconfig = CreationConfig.readconfig(self.path_config)
         if self.envconfig.recreate:
@@ -266,11 +284,11 @@ class VirtualEnv(object):
         alwayscopy = self.envconfig.alwayscopy
         deps = []
         for dep in self.get_resolved_dependencies():
-            dep_name_md5 = getdigest(dep.name)
-            deps.append((dep_name_md5, dep.name))
-        base_resolved_python_md5 = getdigest(base_resolved_python_path)
+            dep_name_sha256 = getdigest(dep.name)
+            deps.append((dep_name_sha256, dep.name))
+        base_resolved_python_sha256 = getdigest(base_resolved_python_path)
         return CreationConfig(
-            base_resolved_python_md5,
+            base_resolved_python_sha256,
             base_resolved_python_path,
             version,
             sitepackages,
@@ -308,10 +326,16 @@ class VirtualEnv(object):
         args = [self.envconfig.envpython, str(setup_py), "--name"]
         env = self._get_os_environ()
         output = action.popen(
-            args, cwd=setupdir, redirect=False, returnout=True, env=env, capture_err=False
+            args,
+            cwd=setupdir,
+            redirect=False,
+            returnout=True,
+            env=env,
+            capture_err=False,
         )
         name = next(
-            (i for i in output.split("\n") if i and not i.startswith("pydev debugger:")), ""
+            (i for i in output.split("\n") if i and not i.startswith("pydev debugger:")),
+            "",
         )
         args = [
             self.envconfig.envpython,
@@ -406,6 +430,9 @@ class VirtualEnv(object):
                 redirect=reporter.verbosity() < reporter.Verbosity.DEBUG,
                 env=env,
             )
+        except KeyboardInterrupt:
+            self.status = "keyboardinterrupt"
+            raise
         finally:
             sys.stdout = old_stdout
 
@@ -421,7 +448,7 @@ class VirtualEnv(object):
                     # equivalent to not setting it at all.
                     reporter.warning(
                         "Discarding $PYTHONPATH from environment, to override "
-                        "specify PYTHONPATH in 'passenv' in your configuration."
+                        "specify PYTHONPATH in 'passenv' in your configuration.",
                     )
                 env.pop("PYTHONPATH")
 
@@ -469,7 +496,7 @@ class VirtualEnv(object):
             env = os.environ.copy()
 
         # in any case we honor per-testenv setenv configuration
-        env.update(self.envconfig.setenv)
+        env.update(self.envconfig.setenv.export())
 
         env["VIRTUAL_ENV"] = str(self.path)
         return env
@@ -495,11 +522,12 @@ class VirtualEnv(object):
                 env = self._get_os_environ(is_test_command=True)
                 # Display PYTHONHASHSEED to assist with reproducibility.
                 action.setactivity(name, "PYTHONHASHSEED={!r}".format(env.get("PYTHONHASHSEED")))
-            for i, argv in enumerate(commands):
+            for i, argv in enumerate(filter(bool, commands)):
                 # have to make strings as _pcall changes argv[0] to a local()
                 # happens if the same environment is invoked twice
                 message = "commands[{}] | {}".format(
-                    i, " ".join([pipes.quote(str(x)) for x in argv])
+                    i,
+                    " ".join([pipes.quote(str(x)) for x in argv]),
                 )
                 action.setactivity(name, message)
                 # check to see if we need to ignore the return code
@@ -555,7 +583,8 @@ class VirtualEnv(object):
         # construct environment variables
         env.pop("VIRTUALENV_PYTHON", None)
         bin_dir = str(self.envconfig.envbindir)
-        env["PATH"] = os.pathsep.join([bin_dir, os.environ["PATH"]])
+        path = self.envconfig.setenv.get("PATH") or os.environ["PATH"]
+        env["PATH"] = os.pathsep.join([bin_dir, path])
         reporter.verbosity2("setting PATH={}".format(env["PATH"]))
 
         # get command
@@ -577,9 +606,14 @@ class VirtualEnv(object):
     def setupenv(self):
         if self.envconfig._missing_subs:
             self.status = (
-                "unresolvable substitution(s): {}. "
+                "unresolvable substitution(s):\n    {}\n"
                 "Environment variables are missing or defined recursively.".format(
-                    ",".join(["'{}'".format(m) for m in self.envconfig._missing_subs])
+                    "\n    ".join(
+                        [
+                            "{}: '{}'".format(section_key, exc.name)
+                            for section_key, exc in sorted(self.envconfig._missing_subs.items())
+                        ],
+                    ),
                 )
             )
             return
@@ -605,6 +639,9 @@ class VirtualEnv(object):
                 status = e
                 if self.envconfig.config.option.skip_missing_interpreters == "true":
                     default_ret_code = 0
+            except KeyboardInterrupt:
+                self.status = "keyboardinterrupt"
+                raise
             if status:
                 str_status = str(status)
                 command_log = envlog.get_commandlog("setup")
@@ -629,7 +666,7 @@ def getdigest(path):
     path = py.path.local(path)
     if not path.check(file=1):
         return "0" * 32
-    return path.computehash()
+    return path.computehash("sha256")
 
 
 def prepend_shebang_interpreter(args):
@@ -669,6 +706,8 @@ def tox_testenv_create(venv, action):
         args.append("--always-copy")
     if not venv.envconfig.download:
         args.append("--no-download")
+    else:
+        args.append("--download")
     # add interpreter explicitly, to prevent using default (virtualenv.ini)
     args.extend(["--python", str(config_interpreter)])
 
@@ -693,12 +732,12 @@ def tox_testenv_create(venv, action):
 
 
 def cleanup_for_venv(venv):
-    within_parallel = PARALLEL_ENV_VAR_KEY in os.environ
+    within_parallel = PARALLEL_ENV_VAR_KEY_PRIVATE in os.environ
     # if the directory exists and it doesn't look like a virtualenv, produce
     # an error
     if venv.path.exists():
         dir_items = set(os.listdir(str(venv.path))) - {".lock", "log"}
-        dir_items = {p for p in dir_items if not p.startswith(".tox-")}
+        dir_items = {p for p in dir_items if not p.startswith(".tox-") or p == ".tox-config1"}
     else:
         dir_items = set()
 
@@ -707,6 +746,8 @@ def cleanup_for_venv(venv):
         not venv.path.exists()
         # does exist, but it's empty => OK
         or not dir_items
+        # tox has marked this as an environment it has created in the past
+        or ".tox-config1" in dir_items
         # it exists and we're on windows with Lib and Scripts => OK
         or (INFO.IS_WIN and dir_items > {"Scripts", "Lib"})
         # non-windows, with lib and bin => OK
@@ -717,7 +758,7 @@ def cleanup_for_venv(venv):
         venv.status = "error"
         reporter.error(
             "cowardly refusing to delete `envdir` (it does not look like a virtualenv): "
-            "{}".format(venv.path)
+            "{}".format(venv.path),
         )
         raise SystemExit(2)
 
